@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005 Sun Microsystems Inc.
+# Copyright 2005-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -56,7 +56,15 @@ Braille display.
 
 import atspi
 import brl
-import brlmon
+try:
+    # This can fail due to gtk not being available.  We want to
+    # be able to recover from that if possible.  The main driver
+    # for this is to allow "orca --text-setup" to work even if
+    # the desktop is not running.
+    #
+    import brlmon
+except:
+    pass
 import debug
 import eventsynthesizer
 import settings
@@ -423,7 +431,12 @@ class Line:
         for region in self.regions:
             if region == _regionWithFocus:
                 focusOffset = len(string)
-            string += region.string
+            if region.string:
+                # [[[TODO: WDW - HACK: Replace unicode ellipses with "..."
+                # The ultimate solution is to get i18n support into
+                # BrlTTY.]]]
+                #
+                string += region.string.replace("\342\200\246", "...")
 
         return [string, focusOffset]
 
@@ -635,27 +648,34 @@ def refresh(panToCursor=True, targetCursorCell=0):
 
     debug.println(debug.LEVEL_INFO, "     VISIBLE:  '%s', cursor=%d" \
                   % (string[startPos:endPos], cursorCell))
-  
+
     brl.writeText(cursorCell, string[startPos:endPos])
 
-    if settings.getSetting(settings.USE_BRAILLE_MONITOR, False):
-	if not monitor:	
-	    monitor = brlmon.BrlMon(_displaySize[0])
-	    monitor.show_all()
+    if settings.enableBrailleMonitor:
+        if not monitor:
+            monitor = brlmon.BrlMon(_displaySize[0])
+            monitor.show_all()
         monitor.writeText(cursorCell, string[startPos:endPos])
+    elif monitor:
+        monitor.destroy()
 
     beginningIsShowing = startPos == 0
     endIsShowing = endPos >= len(string)
 
-def displayRegionsWithFocus(regions, focusedRegion):
-    """Displays a list of regions on a single line, setting focus to the 
-       specified region.
+def displayRegions(regionInfo):
+    """Displays a list of regions on a single line, setting focus to the
+       specified region.  The regionInfo parameter is something that is
+       typically returned by a call to braillegenerator.getBrailleRegions.
 
     Arguments:
-    - regions: a list of regions to display
-    - focusedRegion: the focused region (must be in regions)
+    - regionInfo: a list where the first element is a list of regions
+                  to display and the second element is the region
+                  with focus (must be in the list from element 0)
     """
 
+    regions = regionInfo[0]
+    focusedRegion = regionInfo[1]
+
     clear()
     line = Line()
     for item in regions:
@@ -664,17 +684,6 @@ def displayRegionsWithFocus(regions, focusedRegion):
     setFocus(focusedRegion)
     refresh()
 
-def displayRegions(regions, indexOfFocusRegion=0):
-    """Displays a list of regions on a single line, setting focus to the 
-       specified region.
-
-    Arguments:
-    - regions: a list of regions to display
-    - indexOfFocusRegion: which region in the list should receive focus,
-                          or the first one if this is not specified
-    """
-    displayRegionsWithFocus(region, regions[indexOfFocusRegion])
-
 def displayMessage(message, cursor=-1):
     """Displays a single line, setting the cursor to the given position,
     ensuring that the cursor is in view.
@@ -805,7 +814,7 @@ def _processBrailleEvent(command):
             cursor = (command - 0x100) + _viewport[0]
             lineNum = _viewport[1]
             _lines[lineNum].processCursorKey(cursor)
-	    return True
+            return True
 
     return False
 
