@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005 Sun Microsystems Inc.
+# Copyright 2005-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -25,6 +25,7 @@ import orca.rolenames as rolenames
 import orca.settings as settings
 import orca.speech as speech
 import orca.speechgenerator as speechgenerator
+import orca.util as util
 
 from orca.orca_i18n import _
 
@@ -92,13 +93,14 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
                 # is the way to identify the menu that's duplicated in the
                 # hierarchy)
                 #
-                if len(parent.label) > 0:
+                label = util.getDisplayedLabel(parent)
+                if label and len(label):
                     if (parent.role == rolenames.ROLE_MENU) \
                        and not parent.state.count(\
                                    atspi.Accessibility.STATE_FOCUSABLE):
                         pass
                     else:
-                        utterances.append(parent.label + " " \
+                        utterances.append(label + " " \
                                       + rolenames.getSpeechForRoleName(parent))
                 # Otherwise, we won't speak it if it is a panel with
                 # no name,
@@ -196,7 +198,7 @@ class Script(default.Script):
     def onLinkSelected(self, event):
         text = event.source.text
         hypertext = event.source.hypertext
-        linkIndex = default.getLinkIndex(event.source, text.caretOffset)
+        linkIndex = util.getLinkIndex(event.source, text.caretOffset)
         
         if linkIndex >= 0:
             link = hypertext.getLink(linkIndex)
