@@ -24,16 +24,70 @@ case of gdm) or doesn't have the specified attribute.
 
 import re
 import sys
+try:
+    # This can fail due to gtk not being available.  We want to
+    # be able to recover from that if possible.  The main driver
+    # for this is to allow "orca --text-setup" to work even if
+    # the desktop is not running.
+    #
+    import gtk.gdk
+    screenWidth = gtk.gdk.screen_get_default().get_width()
+    screenHeight = gtk.gdk.screen_get_default().get_height()
+except:
+    # Just default to something that hopefully everybody has.
+    #
+    screenWidth = 640
+    screenHeight = 480
 
 import debug
 from acss import ACSS
 from orca_i18n import _           # for gettext support
 
-#########################################################################
-#                                                                       #
-# String constants and default values for the common attributes.        #
-#                                                                       #
-#########################################################################
+# These are the settings that Orca supports the user customizing.
+#
+userCustomizableSettings = [
+    "orcaModifierKeys",
+    "enableSpeech",
+    "speechServerFactory",
+    "speechServerInfo",
+    "voices",
+    "speechVerbosityLevel",
+    "enableEchoByWord",
+    "enableKeyEcho",
+    "enablePrintableKeys",
+    "enableModifierKeys",
+    "enableLockingKeys",
+    "enableFunctionKeys",
+    "enableActionKeys",
+    "enableBraille",
+    "brailleVerbosityLevel",
+    "brailleRolenameStyle",
+    "enableBrailleMonitor",
+    "enableMagnifier",
+    "enableMagCursor",
+    "enableMagCursorExplicitSize",
+    "magCursorSize",
+    "magCursorColor",
+    "enableMagCrossHair",
+    "enableMagCrossHairClip",
+    "magCrossHairSize",
+    "magZoomerLeft",
+    "magZoomerRight",
+    "magZoomerTop",
+    "magZoomerBottom",
+    "magZoomFactor",
+    "enableMagZoomerColorInversion",
+    "magSmoothingMode",
+    "magMouseTrackingMode",
+    "verbalizePunctuationStyle",
+]
+
+# The name of the modules that hold the user interface for setting
+# Orca preferences.  Each module is expected to have the method,
+# showPreferencesUI, which will prompt the user for preferences.
+#
+guiPreferencesModule    = "orca_gui_prefs"
+consolePreferencesModule= "orca_console_prefs"
 
 # A list of keys that can serve as the Orca modifier key.  The list is
 # so we can provide better cross platform support (e.g., Sun keyboard
@@ -57,6 +111,13 @@ BRAILLE_ROLENAME_STYLE_SHORT = 0 # three letter abbreviations
 BRAILLE_ROLENAME_STYLE_LONG  = 1 # full rolename
 brailleRolenameStyle    = BRAILLE_ROLENAME_STYLE_LONG
 
+# Speech punctuation levels (see verbalizePunctuationStyle).
+#
+PUNCTUATION_STYLE_NONE = 0
+PUNCTUATION_STYLE_SOME = 1
+PUNCTUATION_STYLE_ALL  = 2
+verbalizePunctuationStyle = PUNCTUATION_STYLE_ALL
+
 # The absolue amount to change the speech rate when
 # increasing or decreasing speech.  This is a numerical
 # value that represents an ACSS rate value.
@@ -68,6 +129,10 @@ speechRateDelta         = 5
 enableSpeech            = True
 enableSpeechCallbacks   = True
 
+# If True, speech has been temporarily silenced.
+#
+silenceSpeech           = False
+
 # Settings that apply to the particular speech engine to
 # use as well details on the default voices to use.
 #
@@ -95,7 +160,64 @@ enableBrailleMonitor    = False
 
 # If True, enable magnification.
 #
-enableMagnifier         = False
+enableMagnifier                  = False
+
+# If True, show the magnification cursor.
+#
+enableMagCursor                  = True
+
+# If True, allow an explicit size for the magnification cursor.
+#
+enableMagCursorExplicitSize      = False
+
+# Size of the magnification cursor (in pixels).
+#
+magCursorSize                    = 32
+
+# Magnification cursor color value (hex color spec).
+#
+magCursorColor                   = '#000000'
+
+# If True, show the magnification cross-hairs.
+#
+enableMagCrossHair               = True
+
+# If True, enable magnification cross-hair clipping.
+#
+enableMagCrossHairClip           = False
+
+# Size of the magnification cross-hairs (in pixels).
+#
+magCrossHairSize                 = 16
+
+# Magnification zoomer region placement.
+#
+magZoomerLeft                    = screenWidth / 2
+magZoomerRight                   = screenWidth
+magZoomerTop                     = 0
+magZoomerBottom                  = screenHeight
+
+# Magnification zoom factor.
+#
+magZoomFactor                    = 4.0
+
+# If True, invert the magnification zoomer colors.
+#
+enableMagZoomerColorInversion    = False
+
+# Magnification smoothing mode (see magSmoothingMode).
+#
+MAG_SMOOTHING_MODE_BILINEAR      = 0
+MAG_SMOOTHING_MODE_NONE          = 1
+magSmoothingMode                 = MAG_SMOOTHING_MODE_BILINEAR
+
+# Magnification mouse tracking mode (see magMouseTrackingMode).
+#
+MAG_MOUSE_TRACKING_MODE_CENTERED     = 0
+MAG_MOUSE_TRACKING_MODE_NONE         = 1
+MAG_MOUSE_TRACKING_MODE_PROPORTIONAL = 2
+MAG_MOUSE_TRACKING_MODE_PUSH         = 3
+magMouseTrackingMode                 = MAG_MOUSE_TRACKING_MODE_CENTERED
 
 # if True, enable word echo.
 # Note that it is allowable for both enableEchoByWord and enableKeyEcho
@@ -160,6 +282,14 @@ cacheValues             = True
 #
 learnModeEnabled        = False
 
+# Assists with dealing with CORBA COMM_FAILURES.  A failure doesn't
+# always mean an object disappeared - there just might be a network
+# glitch.  So, on COMM_FAILURES, we might retry a few times before
+# giving up on an object.
+#
+commFailureWaitTime     = 0.1
+commFailureAttemptLimit = 5
+
 # Which packages to search, and the order in which to search,
 # for custom scripts.  These packages are expected to be on
 # the PYTHONPATH and/or subpackages of the "orca" package.
@@ -201,6 +331,9 @@ def getScriptModuleName(app):
     - app: the application to find a script module name for
     """
 
+    if not app.name:
+        return None
+
     for mapping in _scriptMappings:
         regExpression = mapping[0]
         moduleName = mapping[1]
