@@ -27,13 +27,12 @@ import math
 
 import atspi
 import debug
-
 import orca
 import rolenames
 import settings
+import util
 
 from orca_i18n import _                          # for gettext support
-from rolenames import getSpeechForRoleName       # localized role names
 
 class SpeechGenerator:
     """Takes accessible objects and produces a string to speak for
@@ -144,7 +143,7 @@ class SpeechGenerator:
         self.speechGenerators[rolenames.ROLE_WINDOW]              = \
              self._getSpeechForWindow
 
-    def _getSpeechForAccelerator(self, obj):
+    def _getSpeechForObjectAccelerator(self, obj):
         """Returns a list of utterances that describes the keyboard
         accelerator (and possibly shortcut) for the given object.
 
@@ -173,7 +172,7 @@ class SpeechGenerator:
 
         return utterances
 
-    def _getSpeechForAvailability(self, obj):
+    def _getSpeechForObjectAvailability(self, obj):
         """Returns a list of utterances that describes the availability
         of the given object.
 
@@ -182,40 +181,41 @@ class SpeechGenerator:
 
         Returns a list of utterances to be spoken.
         """
-
-        utterances = []
         if obj.state.count(atspi.Accessibility.STATE_SENSITIVE) == 0:
-            utterances.append(_("grayed"))
+            return [_("grayed")]
+        else:
+            return []
 
-        return utterances
-
-    def _getSpeechForLabelAndRole(self, obj):
-        """Returns the list of utteranaces to be spoken for the object's
-        label and role.
-
-        Arguments:
-        - obj: an Accessible
-
-        Returns a list of utterances to be spoken for the label and role.
-        """
-
-        verbosity = settings.speechVerbosityLevel
-
-        text = obj.label
-
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if (obj.label == None) or (obj.role != rolenames.ROLE_UNKNOWN):
-                text += " " + getSpeechForRoleName(obj)
-
-        return [text]
+    def _getSpeechForObjectLabel(self, obj):
+        label = util.getDisplayedLabel(obj)
+        if label:
+            return [label]
+        else:
+            return []
+        
+    def _getSpeechForObjectName(self, obj):
+        name = util.getDisplayedText(obj)
+        if name:
+            return [name]
+        elif obj.description:
+            return [obj.description]
+        else:            
+            return []
 
+    def _getSpeechForObjectRole(self, obj):
+        if (settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE)\
+           and (obj.role != rolenames.ROLE_UNKNOWN):
+            return [rolenames.getSpeechForRoleName(obj)]
+        else:
+            return []
+        
     def _debugGenerator(self, generatorName, obj, already_focused, utterances):
         """Prints debug.LEVEL_FINER information regarding the speech generator.
 
         Arguments:
         - generatorName: the name of the generator
         - obj: the object being presented
-        - already_focused: boolean stating if object just received focus
+        - already_focused: False if object just received focus
         - utterances: the generated text
         """
 
@@ -241,11 +241,11 @@ class SpeechGenerator:
 
         The default speech will be of the following form:
 
-        label [role] [accelerator] [availability]
+        label name role availability
 
         Arguments:
         - obj: an Accessible
-        - already_focused: True if object just received focus; False otherwise
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -253,9 +253,12 @@ class SpeechGenerator:
         utterances = []
 
         if not already_focused:
-            utterances.extend(self._getSpeechForLabelAndRole(obj))
-            utterances.extend(self._getSpeechForAvailability(obj))
-
+            utterances.extend(self._getSpeechForObjectLabel(obj))
+            utterances.extend(self._getSpeechForObjectName(obj))
+            utterances.extend(self._getSpeechForObjectRole(obj))
+            
+        utterances.extend(self._getSpeechForObjectAvailability(obj))
+        
         self._debugGenerator("_getDefaultSpeech",
                              obj,
                              already_focused,
@@ -269,12 +272,15 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the Accessible dialog
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances be spoken.
         """
 
-        utterances = self._getSpeechForLabelAndRole(obj)
+        utterances = []
+        utterances.extend(self._getSpeechForObjectLabel(obj))
+        utterances.extend(self._getSpeechForObjectName(obj))
+        utterances.extend(self._getSpeechForObjectRole(obj))
 
         # Find all the unrelated labels in the dialog and speak them.
         #
@@ -294,15 +300,15 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the animation
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken.
         """
 
-        utterances = self._getSpeechForLabelAndRole(obj)
-
-        if obj.description:
-            utterances.append(obj.description)
+        utterances = []
+        utterances.extend(self._getSpeechForObjectLabel(obj))
+        utterances.extend(self._getSpeechForObjectName(obj))
+        utterances.extend(self._getSpeechForObjectRole(obj))
 
         self._debugGenerator("_getSpeechForAnimation",
                              obj,
@@ -316,7 +322,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the arrow
-        - already_focused: True if object just received focus; False otherwise
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -325,7 +331,7 @@ class SpeechGenerator:
         # 319744.]]]
         # text = arrow direction (left, right, up, down)
         #
-        utterances = self._getSpeechForLabelAndRole(obj)
+        utterances = self._getDefaultSpeech(obj, already_focused)
 
         self._debugGenerator("_getSpeechForArrow",
                              obj,
@@ -340,7 +346,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the check box
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -350,18 +356,20 @@ class SpeechGenerator:
             # If it's not already focused, say it's name
             #
             if not already_focused:
-                tmp = self._getSpeechForLabelAndRole(obj)
-                tmp.append(_("checked"))
-                utterances.extend(tmp)
-                utterances.extend(self._getSpeechForAvailability(obj))
+                utterances.extend(self._getSpeechForObjectLabel(obj))
+                utterances.extend(self._getSpeechForObjectName(obj))
+                utterances.extend(self._getSpeechForObjectRole(obj))
+                utterances.append(_("checked"))
+                utterances.extend(self._getSpeechForObjectAvailability(obj))
             else:
                 utterances.append(_("checked"))
         else:
             if not already_focused:
-                tmp = self._getSpeechForLabelAndRole(obj)
-                tmp.append(_("not checked"))
-                utterances.extend(tmp)
-                utterances.extend(self._getSpeechForAvailability(obj))
+                utterances.extend(self._getSpeechForObjectLabel(obj))
+                utterances.extend(self._getSpeechForObjectName(obj))
+                utterances.extend(self._getSpeechForObjectRole(obj))
+                utterances.append(_("not checked"))
+                utterances.extend(self._getSpeechForObjectAvailability(obj))
             else:
                 utterances.append(_("not checked"))
 
@@ -378,18 +386,16 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the check menu item
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
 
-        verbosity = settings.speechVerbosityLevel
-
         utterances = self._getSpeechForCheckBox(obj, already_focused)
 
-        if (verbosity == settings.VERBOSITY_LEVEL_VERBOSE) \
+        if (settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE)\
            and not already_focused:
-            utterances.extend(self._getSpeechForAccelerator(obj))
+            utterances.extend(self._getSpeechForObjectAccelerator(obj))
 
         self._debugGenerator("_getSpeechForCheckMenuItem",
                              obj,
@@ -403,7 +409,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the column header
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -423,70 +429,23 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the combo box
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
 
-        verbosity = settings.speechVerbosityLevel
-
         utterances = []
-        if not already_focused:
-            utterances.append(obj.label)
-
-        # Find the text displayed in the combo box.  This is either:
-        #
-        # 1) The last text object that's a child of the combo box
-        # 2) The selected child of the combo box.
-        # 3) The contents of the text of the combo box itself when
-        #    treated as a text object.
-        #
-        # Preference is given to #1, if it exists.
-        #
-        # If the label of the combo box is the same as the utterance for
-        # the child object, then this utterance is only spoken once.
-        #
-        # [[[TODO: WDW - Combo boxes are complex beasts.  This algorithm
-        # needs serious work.  Logged as bugzilla bug 319745.]]]
-        #
-        textObj = None
-        for i in range(0, obj.childCount):
-            debug.println(debug.LEVEL_FINEST,
-                          "speechgenerator._getSpeechForComboBox " \
-                          + "looking at child %d" % i)
-            child = obj.child(i)
-            if child.role == rolenames.ROLE_TEXT:
-                textObj = child
-
-        if textObj:
-            result = atspi.getTextLineAtCaret(textObj)
-            line = result[0]
-            if line != obj.label:
-                utterances.append(line)
-        else:
-            selectedItem = None
-            comboSelection = obj.selection
-            if comboSelection and comboSelection.nSelectedChildren > 0:
-                selectedItem = atspi.Accessible.makeAccessible(\
-                    comboSelection.getSelectedChild(0))
-            if selectedItem:
-                if selectedItem.label != obj.label:
-                    utterances.append(selectedItem.label)
-            else:
-                result = atspi.getTextLineAtCaret(obj)
-                selectedText = result[0]
-                if len(selectedText) > 0:
-                    if selectedText != obj.label:
-                        utterances.append(selectedText)
-                else:
-                    debug.println(
-                        debug.LEVEL_SEVERE,
-                        "ERROR: Could not find selected item for combo box.")
 
         if not already_focused:
-            if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-                utterances.append(getSpeechForRoleName(obj))
+            utterances.extend(self._getSpeechForObjectLabel(obj))
 
+        utterances.extend(self._getSpeechForObjectName(obj))
+
+        if not already_focused:
+            utterances.extend(self._getSpeechForObjectRole(obj))
+            
+        utterances.extend(self._getSpeechForObjectAvailability(obj))
+        
         self._debugGenerator("_getSpeechForComboBox",
                              obj,
                              already_focused,
@@ -499,7 +458,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the desktop icon
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -518,7 +477,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the dial
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -540,7 +499,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the dialog box
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -559,7 +518,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the dial
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -578,7 +537,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the frame
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -598,7 +557,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the dial
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -617,7 +576,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the icon
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -628,35 +587,31 @@ class SpeechGenerator:
         # then a custom script for nautilus needs to be written to remove the
         # availability.]]]
         #
-        verbosity = settings.speechVerbosityLevel
-
-        text = obj.label
+        utterances = []
+        utterances.extend(self._getSpeechForObjectLabel(obj))
+        utterances.extend(self._getSpeechForObjectName(obj))
 
         if obj.image:
             description = obj.image.imageDescription
-            if len(description):
-                if len(text):
-                    text += " "
-                text += description
+            if description and len(description):
+                utterances.append(description)
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if len(text):
-                text += " "
-            text += getSpeechForRoleName(obj)
+        if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
+            utterances.append(rolenames.getSpeechForRoleName(obj))
 
         self._debugGenerator("_getSpeechForIcon",
                              obj,
                              already_focused,
-                             [text])
+                             utterances)
 
-        return [text]
+        return utterances
 
     def _getSpeechForImage(self, obj, already_focused):
         """Get the speech for an image.
 
         Arguments:
         - obj: the image
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -675,7 +630,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the label
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -694,7 +649,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the list
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -716,19 +671,17 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the menu
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
 
-        verbosity = settings.speechVerbosityLevel
-
         utterances = self._getDefaultSpeech(obj, already_focused)
 
         if (obj == orca.locusOfFocus) \
-               and (verbosity == settings.VERBOSITY_LEVEL_VERBOSE):
-            utterances.extend(self._getSpeechForAvailability(obj))
-            utterances.extend(self._getSpeechForAccelerator(obj))
+               and (settings.speechVerbosityLevel \
+                    == settings.VERBOSITY_LEVEL_VERBOSE):
+            utterances.extend(self._getSpeechForObjectAccelerator(obj))
 
         self._debugGenerator("_getSpeechForMenu",
                              obj,
@@ -742,7 +695,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the menu bar
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -761,20 +714,17 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the menu item
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
 
-        verbosity = settings.speechVerbosityLevel
-
-        utterances = [obj.label]
-
         # No need to say "menu item" because we already know that.
         #
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            utterances.extend(self._getSpeechForAvailability(obj))
-            utterances.extend(self._getSpeechForAccelerator(obj))
+        utterances = self._getSpeechForObjectName(obj)
+        if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
+            utterances.extend(self._getSpeechForObjectAccelerator(obj))
 
         self._debugGenerator("_getSpeechForMenuItem",
                              obj,
@@ -788,7 +738,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the text component
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -797,10 +747,15 @@ class SpeechGenerator:
         # areas, such as those in yelp, come up as insensitive though
         # they really are ineditable.]]]
         #
-        utterances = self._getSpeechForLabelAndRole(obj)
+        utterances = []
+        utterances.extend(self._getSpeechForObjectLabel(obj))
+        if len(utterances) == 0:
+            if obj.name and (len(obj.name)):
+                utterances.append(obj.name)
+        utterances.extend(self._getSpeechForObjectRole(obj))
 
-        result = atspi.getTextLineAtCaret(obj)
-        utterances.append(result[0])
+        [text, startOffset, endOffset] = atspi.getTextLineAtCaret(obj)
+        utterances.append(text)
 
         self._debugGenerator("_getSpeechForText",
                              obj,
@@ -814,7 +769,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the option pane
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -833,7 +788,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the page tab
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -852,16 +807,17 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the page tab list
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
 
         utterances = self._getDefaultSpeech(obj, already_focused)
-        if obj.childCount == 1:
-            utterances.append(_("one tab"))
-        else:
-            utterances.append(("%d " % obj.childCount) + _("tabs"))
+
+        #if obj.childCount == 1:
+        #    utterances.append(_("one tab"))
+        #else:
+        #    utterances.append(("%d " % obj.childCount) + _("tabs"))
 
         self._debugGenerator("_getSpeechForPageTabList",
                              obj,
@@ -876,7 +832,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the progress bar
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -884,11 +840,14 @@ class SpeechGenerator:
         percentage = ("%d" % obj.value.currentValue) + " " \
                      + _("percent") + ". "
 
+        utterances = []
+
         if not already_focused:
-            utterances = self._getSpeechForLabelAndRole(obj)
-            utterances.append(percentage)
-        else:
-            utterances = [percentage]
+            utterances.extend(self._getSpeechForObjectLabel(obj))
+            utterances.extend(self._getSpeechForObjectName(obj))
+            utterances.extend(self._getSpeechForObjectRole(obj))
+            
+        utterances.append(percentage)
 
         self._debugGenerator("_getSpeechForProgressBar",
                              obj,
@@ -902,7 +861,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the push button
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -922,7 +881,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the check box
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -932,18 +891,24 @@ class SpeechGenerator:
             # If it's not already focused, say it's name
             #
             if not already_focused:
-                tmp = self._getSpeechForLabelAndRole(obj)
-                tmp.append(_("selected"))
-                utterances.extend(tmp)
-                utterances.extend(self._getSpeechForAvailability(obj))
+                # The label is handled as a context in default.py
+                #
+                #utterances.extend(self._getSpeechForObjectLabel(obj))
+                utterances.extend(self._getSpeechForObjectName(obj))
+                utterances.extend(self._getSpeechForObjectRole(obj))
+                utterances.append(_("selected"))
+                utterances.extend(self._getSpeechForObjectAvailability(obj))
             else:
                 utterances.append(_("selected"))
         else:
             if not already_focused:
-                tmp = self._getSpeechForLabelAndRole(obj)
-                tmp.append(_("not selected"))
-                utterances.extend(tmp)
-                utterances.extend(self._getSpeechForAvailability(obj))
+                # The label is handled as a context in default.py
+                #
+                #utterances.extend(self._getSpeechForObjectLabel(obj))
+                utterances.extend(self._getSpeechForObjectName(obj))
+                utterances.extend(self._getSpeechForObjectRole(obj))
+                utterances.append(_("not selected"))
+                utterances.extend(self._getSpeechForObjectAvailability(obj))
             else:
                 utterances.append(_("not selected"))
 
@@ -960,18 +925,16 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the check menu item
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
 
-        verbosity = settings.speechVerbosityLevel
-
         utterances = self._getSpeechForRadioButton(obj, False)
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            utterances.extend(self._getSpeechForAvailability(obj))
-            utterances.extend(self._getSpeechForAccelerator(obj))
+        if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
+            utterances.extend(self._getSpeechForObjectAccelerator(obj))
 
         self._debugGenerator("_getSpeechForRadioMenuItem",
                              obj,
@@ -985,7 +948,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the column header
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -1004,12 +967,13 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the scroll bar
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
 
-        # [[[TODO: WDW - want to get orientation. Logged as bugzilla bug
+        # [[[TODO: WDW - want to get orientation and maybe the
+        # percentage scrolled so far. Logged as bugzilla bug
         # 319744.]]]
         #
         utterances = self._getDefaultSpeech(obj, already_focused)
@@ -1027,7 +991,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the slider
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -1067,11 +1031,15 @@ class SpeechGenerator:
         if already_focused:
             utterances = [valueString]
         else:
-            utterances = self._getSpeechForLabelAndRole(obj)
+            utterances = []
+            utterances.extend(self._getSpeechForObjectLabel(obj))
+            # Ignore the text on the slider.
+            #utterances.extend(self._getSpeechForObjectName(obj))
+            utterances.extend(self._getSpeechForObjectRole(obj))
             utterances.append(valueString)
-            utterances.extend(self._getSpeechForAvailability(obj))
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
 
-        self._debugGenerator("_getSpeechForProgressBar",
+        self._debugGenerator("_getSpeechForSlider",
                              obj,
                              already_focused,
                              utterances)
@@ -1084,19 +1052,16 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the spin button
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
 
         if already_focused:
-            utterances = []
+            utterances = [util.getDisplayedText(obj)]
         else:
             utterances = self._getDefaultSpeech(obj, already_focused)
 
-        result = atspi.getTextLineAtCaret(obj)
-        utterances.append(result[0])
-
         self._debugGenerator("_getSpeechForSpinButton",
                              obj,
                              already_focused,
@@ -1109,7 +1074,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the split pane
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -1128,7 +1093,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the table
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -1147,7 +1112,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the table
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -1171,16 +1136,10 @@ class SpeechGenerator:
                                                             already_focused)
                     obj.role = rolenames.ROLE_TABLE_CELL
                     break
-                #elif action.getName(i) == "edit":
-                #    utterances = self._getSpeechForText(obj, True)
-                #    break
 
         if (len(utterances) == 0) and (not already_focused):
-            utterances = [obj.label]
-            if obj.text:
-                text = obj.text.getText(0, -1)
-                if obj.label != text:
-                    utterances.append(text)
+            utterances.append(util.getDisplayedText(\
+                util.getRealActiveDescendant(obj)))
 
         # [[[TODO: WDW - HACK attempt to determine if this is a node;
         # if so, describe its state.]]]
@@ -1203,7 +1162,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the table column header
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -1222,7 +1181,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the table row header
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -1241,14 +1200,12 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the tear off menu item
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
 
-        verbosity = settings.speechVerbosityLevel
-
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
+        if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
             utterances = [_("tear off menu item")]
         else:
             utterances = [_("tear off")]
@@ -1265,24 +1222,19 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the terminal
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
 
-        verbosity = settings.speechVerbosityLevel
-
-        label = None
+        title = None
         frame = atspi.getFrame(obj)
         if frame:
-            label = frame.name
-        if not label:
-            label = obj.label
+            title = frame.name
+        if not title:
+            title = util.getDisplayedLabel(obj)
 
-        utterances = [label]
-
-        #if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-        #    utterances.append(getSpeechForRoleName(obj))
+        utterances = [title]
 
         self._debugGenerator("_getSpeechForTerminal",
                              obj,
@@ -1297,7 +1249,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the check box
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -1307,18 +1259,20 @@ class SpeechGenerator:
             # If it's not already focused, say it's name
             #
             if not already_focused:
-                tmp = self._getSpeechForLabelAndRole(obj)
-                tmp.append(_("pressed"))
-                utterances.extend(tmp)
-                utterances.extend(self._getSpeechForAvailability(obj))
+                utterances.extend(self._getSpeechForObjectLabel(obj))
+                utterances.extend(self._getSpeechForObjectName(obj))
+                utterances.extend(self._getSpeechForObjectRole(obj))
+                utterances.append(_("pressed"))
+                utterances.extend(self._getSpeechForObjectAvailability(obj))
             else:
                 utterances.append(_("pressed"))
         else:
             if not already_focused:
-                tmp = self._getSpeechForLabelAndRole(obj)
-                tmp.append(_("not pressed"))
-                utterances.extend(tmp)
-                utterances.extend(self._getSpeechForAvailability(obj))
+                utterances.extend(self._getSpeechForObjectLabel(obj))
+                utterances.extend(self._getSpeechForObjectName(obj))
+                utterances.extend(self._getSpeechForObjectRole(obj))
+                utterances.append(_("not pressed"))
+                utterances.extend(self._getSpeechForObjectAvailability(obj))
             else:
                 utterances.append(_("not pressed"))
 
@@ -1334,7 +1288,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the tool bar
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -1353,14 +1307,14 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the tree
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
 
         utterances = self._getDefaultSpeech(obj, already_focused)
 
-        self._debugGenerator("_getSpeechForTreeTable",
+        self._debugGenerator("_getSpeechForTree",
                              obj,
                              already_focused,
                              utterances)
@@ -1372,7 +1326,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the tree table
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -1391,7 +1345,7 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the window
-        - already_focused: if False, the obj just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
@@ -1413,17 +1367,17 @@ class SpeechGenerator:
 
         Arguments:
         - obj: the object
-        - already_focused: boolean stating if object just received focus
+        - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken.
         """
-
+        
         if self.speechGenerators.has_key(obj.role):
             generator = self.speechGenerators[obj.role]
         else:
             generator = self._getDefaultSpeech
 
-        return generator(obj, already_focused)
+        return [" ".join(generator(obj, already_focused))]
 
     def getSpeechContext(self, obj, stopAncestor=None):
         """Get the speech that describes the names and role of
@@ -1470,14 +1424,28 @@ class SpeechGenerator:
                 and (parent.role != rolenames.ROLE_SPLIT_PANE) \
                 and (parent.role != rolenames.ROLE_SCROLL_PANE) \
                 and (parent.role != rolenames.ROLE_UNKNOWN):
-                if len(parent.label) > 0:
-                    utterances.append(parent.label + " " \
-                                      + getSpeechForRoleName(parent))
-                elif parent.role != rolenames.ROLE_PANEL:
-                    utterances.append(getSpeechForRoleName(parent))
 
+                # Announce the label and text of the object in the hierarchy.
+                #
+                text = util.getDisplayedText(parent)
+                label = util.getDisplayedLabel(parent)
+
+                # Don't announce unlabelled panels.
+                #
+                if parent.role == rolenames.ROLE_PANEL \
+                   and (((not label) or (len(label) == 0) \
+                         or (not text) or (len(text) == 0))):
+                    pass
+                else:
+                    utterances.append(rolenames.getSpeechForRoleName(parent))
+
+                if text and len(text):
+                    utterances.append(text)
+                if label and len(label):
+                    utterances.append(label)
+                    
             parent = parent.parent
 
         utterances.reverse()
 
-        return utterances
+        return [" ".join(utterances)]
